package com.tencent.liteav.demo.superplayer.database.repo

import android.util.Log
import com.tencent.liteav.demo.superplayer.database.dao.CountDownDao
import com.tencent.liteav.demo.superplayer.database.dao.HistoryDao
import com.tencent.liteav.demo.superplayer.database.entity.CountDown
import com.tencent.liteav.demo.superplayer.database.entity.History
import kotlinx.coroutines.flow.Flow
import java.text.SimpleDateFormat
import java.util.*

public class PlayerRepository(
    private val historyDao: HistoryDao,
    private val countDownDao: CountDownDao,
) {
    companion object {
        val dateFormat = SimpleDateFormat("yyyy-MM-dd", Locale.SIMPLIFIED_CHINESE)
    }

    /**
     * 定时相关的数据库操作
     */
    fun insertCountDown(countDown: CountDown) {
        countDownDao.insertCountDown(countDown)
    }

    fun deleteCountDown(courseId: String) {
        countDownDao.deleteCountDown(courseId)
    }

    fun findHasCountDown(courseId: String): CountDown? {
        return countDownDao.queryCountDown(courseId)
    }


    /**
     * 历史记录有关的操作
     */
    fun insertHistory(history: History) {
        historyDao.insertHistory(history)
    }

    fun queryTodayEpisodeHistory(
        courseId: String,
        countDown: String,
    ): Int {
        val date = dateFormat.format(Date())
        return historyDao.queryTodayEpisodeHistory(courseId, countDown, date)
    }

    fun queryTodayDurationHistory(
        courseId: String,
        countDown: String,
    ): List<History> {
        val date = dateFormat.format(Date())
        return historyDao.queryTodayDurationHistory(courseId, countDown, date)
    }
}